@@ -11,7 +11,16 @@
 * Mods:    Vince Weaver
 *          vweaver1@eecs.utk.edu
 * Mods:	   Philip Mucci
-*	   mucci@eecs.utk.edu */
+*	   mucci@eecs.utk.edu
+* Mods:    Gary Mohr
+*          gary.mohr@bull.com
+*          Modified the perf_event component to use PFM_OS_PERF_EVENT_EXT mode in libpfm4.
+*          This adds several new event masks, including cpu=, u=, and k= which give the user
+*          the ability to set cpu number to use or control the domain (user, kernel, or both)
+*          in which the counter should be incremented.  These are event masks so it is now 
+*          possible to have multiple events in the same event set that count activity from 
+*          differennt cpu's or count activity in different domains.
+*/
 
 
 #include <fcntl.h>
@@ -57,7 +66,11 @@ papi_vector_t _perf_event_vector;
 
 /* Globals */
 struct native_event_table_t perf_native_event_table;
-int our_cidx;
+static int our_cidx;
+int
+_pe_libpfm4_get_cidx() {
+	return our_cidx;
+}
 
 /* These sentinels tell _pe_set_overflow() how to set the */
 /* wakeup_events field in the event descriptor record.        */
@@ -79,6 +92,8 @@ int our_cidx;
 #define PAPI_REFRESH_VALUE 1
 #endif
 
+static int _pe_set_domain( hwd_control_state_t *ctl, int domain);
+
 /* Check for processor support */
 /* Can be used for generic checking, though in general we only     */
 /* check for pentium4 here because support was broken for multiple */
@@ -300,11 +315,10 @@ sys_perf_event_open( struct perf_event_attr *hw_event, pid_t pid, int cpu,
 {
 	int ret;
 
-   SUBDBG("sys_perf_event_open(%p,%d,%d,%d,%lx\n",hw_event,pid,cpu,group_fd,flags);
+   SUBDBG("sys_perf_event_open(hw_event: %p, pid: %d, cpu: %d, group_fd: %d, flags: %lx\n", hw_event, pid, cpu, group_fd, flags);
    SUBDBG("   type: %d\n",hw_event->type);
    SUBDBG("   size: %d\n",hw_event->size);
-   SUBDBG("   config: %"PRIx64" (%"PRIu64")\n",hw_event->config,
-	  hw_event->config);
+   SUBDBG("   config: %"PRIx64" (%"PRIu64")\n",hw_event->config, hw_event->config);
    SUBDBG("   sample_period: %"PRIu64"\n",hw_event->sample_period);
    SUBDBG("   sample_type: %"PRIu64"\n",hw_event->sample_type);
    SUBDBG("   read_format: %"PRIu64"\n",hw_event->read_format);
@@ -323,6 +337,21 @@ sys_perf_event_open( struct perf_event_attr *hw_event, pid_t pid, int cpu,
    SUBDBG("   enable_on_exec: %d\n",hw_event->enable_on_exec);
    SUBDBG("   task: %d\n",hw_event->task);
    SUBDBG("   watermark: %d\n",hw_event->watermark);
+   SUBDBG("   precise_ip: %d\n",hw_event->precise_ip);
+   SUBDBG("   mmap_data: %d\n",hw_event->mmap_data);
+   SUBDBG("   sample_id_all: %d\n",hw_event->sample_id_all);
+   SUBDBG("   exclude_host: %d\n",hw_event->exclude_host);
+   SUBDBG("   exclude_guest: %d\n",hw_event->exclude_guest);
+   SUBDBG("   exclude_callchain_kernel: %d\n",hw_event->exclude_callchain_kernel);
+   SUBDBG("   exclude_callchain_user: %d\n",hw_event->exclude_callchain_user);
+   SUBDBG("   wakeup_events: %"PRIx32" (%"PRIu32")\n", hw_event->wakeup_events, hw_event->wakeup_events);
+   SUBDBG("   bp_type: %"PRIx32" (%"PRIu32")\n", hw_event->bp_type, hw_event->bp_type);
+   SUBDBG("   config1: %"PRIx64" (%"PRIu64")\n", hw_event->config1, hw_event->config1);
+   SUBDBG("   config2: %"PRIx64" (%"PRIu64")\n", hw_event->config2, hw_event->config2);
+   SUBDBG("   branch_sample_type: %"PRIx64" (%"PRIu64")\n", hw_event->branch_sample_type, hw_event->branch_sample_type);
+   SUBDBG("   sample_regs_user: %"PRIx64" (%"PRIu64")\n", hw_event->sample_regs_user, hw_event->sample_regs_user);
+   SUBDBG("   sample_stack_user: %"PRIx32" (%"PRIu32")\n", hw_event->sample_stack_user, hw_event->sample_stack_user);
+
 	ret =
 		syscall( __NR_perf_event_open, hw_event, pid, cpu, group_fd, flags );
 	SUBDBG("Returned %d %d %s\n",ret,
@@ -470,14 +499,14 @@ check_scheduability( pe_context_t *ctx, pe_control_t *ctl, int idx )
       /* start the event */
       retval = ioctl( group_leader_fd, PERF_EVENT_IOC_ENABLE, NULL );
       if (retval == -1) {
-	 PAPIERROR("ioctl(PERF_EVENT_IOC_ENABLE) failed.\n");
+	 PAPIERROR("ioctl(PERF_EVENT_IOC_ENABLE) failed");
 	 return PAPI_ESYS;
       }
 
       /* stop the event */
       retval = ioctl(group_leader_fd, PERF_EVENT_IOC_DISABLE, NULL );
       if (retval == -1) {
-	 PAPIERROR( "ioctl(PERF_EVENT_IOC_DISABLE) failed.\n" );
+	 PAPIERROR( "ioctl(PERF_EVENT_IOC_DISABLE) failed" );
 	 return PAPI_ESYS;
       }
 
@@ -514,7 +543,7 @@ check_scheduability( pe_context_t *ctx, pe_control_t *ctl, int idx )
 	   retval=ioctl( ctl->events[i].event_fd, PERF_EVENT_IOC_RESET, NULL );
 	   if (retval == -1) {
 	      PAPIERROR( "ioctl(PERF_EVENT_IOC_RESET) #%d/%d %d "
-			 "(fd %d)failed.\n",
+			 "(fd %d)failed",
 			 i,ctl->num_events,idx,ctl->events[i].event_fd);
 	      return PAPI_ESYS;
 	   }
@@ -638,7 +667,7 @@ open_pe_events( pe_context_t *ctx, pe_control_t *ctl )
       /* try to open */
       ctl->events[i].event_fd = sys_perf_event_open( &ctl->events[i].attr, 
 						     pid,
-						     ctl->cpu,
+						     ctl->events[i].cpu,
 			       ctl->events[i].group_leader_fd,
 						     0 /* flags */
 						     );
@@ -655,8 +684,8 @@ open_pe_events( pe_context_t *ctx, pe_control_t *ctl )
 
       SUBDBG ("sys_perf_event_open: tid: %ld, cpu_num: %d,"
               " group_leader/fd: %d, event_fd: %d,"
-              " read_format: %#"PRIu64"\n",
-	      pid, ctl->cpu, ctl->events[i].group_leader_fd, 
+              " read_format: %"PRIu64"\n",
+	      pid, ctl->events[i].cpu, ctl->events[i].group_leader_fd,
 	      ctl->events[i].event_fd, ctl->events[i].attr.read_format);
 
 
@@ -686,7 +715,7 @@ open_pe_events( pe_context_t *ctx, pe_control_t *ctl )
    for ( i = 0; i < ctl->num_events; i++ ) {
 
       /* If sampling is enabled, hook up signal handler */
-      if ( ctl->events[i].attr.sample_period ) {
+      if ((ctl->events[i].attr.sample_period)  &&  (ctl->events[i].nr_mmap_pages > 0)) {
 	 ret = tune_up_fd( ctl, i );
 	 if ( ret != PAPI_OK ) {
 	    /* All of the fds are open, so we need to clean up all of them */
@@ -794,7 +823,7 @@ close_pe_events( pe_context_t *ctx, pe_control_t *ctl )
    if (ctl->num_events!=num_closed) {
       if (ctl->num_events!=(num_closed+events_not_opened)) {
          PAPIERROR("Didn't close all events: "
-		   "Closed %d Not Opened: %d Expected %d\n",
+		   "Closed %d Not Opened: %d Expected %d",
 		   num_closed,events_not_opened,ctl->num_events);
          return PAPI_EBUG;
       }
@@ -815,29 +844,15 @@ close_pe_events( pe_context_t *ctx, pe_control_t *ctl )
 /********************************************************************/
 
 
-/* set the domain. FIXME: perf_events allows per-event control of this. */
-/* we do not handle that yet.                                           */
-int
+/* set the domain. perf_events allows per-event control of this, papi allows it to be set at the event level or at the event set level. */
+/* this will set the event set level domain values but they only get used if no event level domain mask (u= or k=) was specified. */
+static int
 _pe_set_domain( hwd_control_state_t *ctl, int domain)
 {
-
-   int i;
    pe_control_t *pe_ctl = ( pe_control_t *) ctl;
 
-   SUBDBG("old control domain %d, new domain %d\n",
-	  pe_ctl->domain,domain);
-
+   SUBDBG("old control domain %d, new domain %d\n", pe_ctl->domain,domain);
    pe_ctl->domain = domain;
-
-   /* Force the domain on all events */
-   for( i = 0; i < pe_ctl->num_events; i++ ) {
-      pe_ctl->events[i].attr.exclude_user =
-	                !( pe_ctl->domain & PAPI_DOM_USER );
-      pe_ctl->events[i].attr.exclude_kernel =
-			!( pe_ctl->domain & PAPI_DOM_KERNEL );
-      pe_ctl->events[i].attr.exclude_hv =
-			!( pe_ctl->domain & PAPI_DOM_SUPERVISOR );
-   }
    return PAPI_OK;
 }
 
@@ -915,6 +930,8 @@ int
 _pe_read( hwd_context_t *ctx, hwd_control_state_t *ctl,
 	       long long **events, int flags )
 {
+	SUBDBG("ENTER: ctx: %p, ctl: %p, events: %p, flags: %#x\n", ctx, ctl, events, flags);
+
    ( void ) flags;			 /*unused */
    int i, ret = -1;
    pe_context_t *pe_ctx = ( pe_context_t *) ctx;
@@ -964,13 +981,13 @@ _pe_read( hwd_context_t *ctx, hwd_control_state_t *ctl,
 
 	 /* We should read 3 64-bit values from the counter */
 	 if (ret<(signed)(3*sizeof(long long))) {
-	    PAPIERROR("Error!  short read!\n");
+	    PAPIERROR("Error!  short read");
 	    return PAPI_ESYS;
 	 }
 
          SUBDBG("read: fd: %2d, tid: %ld, cpu: %d, ret: %d\n",
 	        pe_ctl->events[i].event_fd,
-		(long)pe_ctl->tid, pe_ctl->cpu, ret);
+		(long)pe_ctl->tid, pe_ctl->events[i].cpu, ret);
          SUBDBG("read: %lld %lld %lld\n",papi_pe_buffer[0],
 	        papi_pe_buffer[1],papi_pe_buffer[2]);
 
@@ -1020,16 +1037,16 @@ _pe_read( hwd_context_t *ctx, hwd_control_state_t *ctl,
 
 	 /* we should read one 64-bit value from each counter */
 	 if (ret!=sizeof(long long)) {
-	    PAPIERROR("Error!  short read!\n");
-	    PAPIERROR("read: fd: %2d, tid: %ld, cpu: %d, ret: %d\n",
+	    PAPIERROR("Error!  short read");
+	    PAPIERROR("read: fd: %2d, tid: %ld, cpu: %d, ret: %d",
 		   pe_ctl->events[i].event_fd,
-		   (long)pe_ctl->tid, pe_ctl->cpu, ret);
+		   (long)pe_ctl->tid, pe_ctl->events[i].cpu, ret);
 	    return PAPI_ESYS;
 	 }
 
          SUBDBG("read: fd: %2d, tid: %ld, cpu: %d, ret: %d\n",
 	        pe_ctl->events[i].event_fd, (long)pe_ctl->tid,
-		pe_ctl->cpu, ret);
+		pe_ctl->events[i].cpu, ret);
          SUBDBG("read: %lld\n",papi_pe_buffer[0]);
 
 	 pe_ctl->counts[i] = papi_pe_buffer[0];
@@ -1042,7 +1059,7 @@ _pe_read( hwd_context_t *ctx, hwd_control_state_t *ctl,
 
    else {
       if (pe_ctl->events[0].group_leader_fd!=-1) {
-	 PAPIERROR("Was expecting group leader!\n");
+	 PAPIERROR("Was expecting group leader");
       }
 
       ret = read( pe_ctl->events[0].event_fd, papi_pe_buffer,
@@ -1056,13 +1073,13 @@ _pe_read( hwd_context_t *ctx, hwd_control_state_t *ctl,
       /* we read 1 64-bit value (number of events) then     */
       /* num_events more 64-bit values that hold the counts */
       if (ret<(signed)((1+pe_ctl->num_events)*sizeof(long long))) {
-	 PAPIERROR("Error! short read!\n");
+	 PAPIERROR("Error! short read");
 	 return PAPI_ESYS;
       }
 
       SUBDBG("read: fd: %2d, tid: %ld, cpu: %d, ret: %d\n",
 	     pe_ctl->events[0].event_fd,
-	     (long)pe_ctl->tid, pe_ctl->cpu, ret);
+	     (long)pe_ctl->tid, pe_ctl->events[0].cpu, ret);
       {
 	 int j;
 	 for(j=0;j<ret/8;j++) {
@@ -1072,12 +1089,12 @@ _pe_read( hwd_context_t *ctx, hwd_control_state_t *ctl,
 
       /* Make sure the kernel agrees with how many events we have */
       if (papi_pe_buffer[0]!=pe_ctl->num_events) {
-	 PAPIERROR("Error!  Wrong number of events!\n");
+	 PAPIERROR("Error!  Wrong number of events");
 	 return PAPI_ESYS;
       }
 
       /* put the count values in their proper location */
-      for(i=0;i<papi_pe_buffer[0];i++) {
+      for(i=0;i<pe_ctl->num_events;i++) {
          pe_ctl->counts[i] = papi_pe_buffer[1+i];
       }
    }
@@ -1106,6 +1123,7 @@ _pe_read( hwd_context_t *ctx, hwd_control_state_t *ctl,
    /* point PAPI to the values we read */
    *events = pe_ctl->counts;
 
+   SUBDBG("EXIT: *events: %p\n", *events);
    return PAPI_OK;
 }
 
@@ -1134,7 +1152,7 @@ _pe_start( hwd_context_t *ctx, hwd_control_state_t *ctl )
 
 	 /* ioctls always return -1 on failure */
          if (ret == -1) {
-            PAPIERROR("ioctl(PERF_EVENT_IOC_ENABLE) failed.\n");
+            PAPIERROR("ioctl(PERF_EVENT_IOC_ENABLE) failed");
             return PAPI_ESYS;
 	 }
 
@@ -1143,7 +1161,7 @@ _pe_start( hwd_context_t *ctx, hwd_control_state_t *ctl )
    }
 
    if (!did_something) {
-      PAPIERROR("Did not enable any counters.\n");
+      PAPIERROR("Did not enable any counters");
       return PAPI_EBUG;
    }
 
@@ -1157,6 +1175,7 @@ _pe_start( hwd_context_t *ctx, hwd_control_state_t *ctl )
 int
 _pe_stop( hwd_context_t *ctx, hwd_control_state_t *ctl )
 {
+	SUBDBG( "ENTER: ctx: %p, ctl: %p\n", ctx, ctl);
 
    int ret;
    int i;
@@ -1178,6 +1197,7 @@ _pe_stop( hwd_context_t *ctx, hwd_control_state_t *ctl )
 
    pe_ctx->state &= ~PERF_EVENTS_RUNNING;
 
+	SUBDBG( "EXIT:\n");
    return PAPI_OK;
 }
 
@@ -1190,7 +1210,12 @@ _pe_update_control_state( hwd_control_state_t *ctl,
 			       NativeInfo_t *native,
 			       int count, hwd_context_t *ctx )
 {
-   int i = 0, ret;
+   SUBDBG( "ENTER: ctl: %p, native: %p, count: %d, ctx: %p\n", ctl, native, count, ctx);
+	int i;
+	int j;
+	int ret;
+	int skipped_events=0;
+	struct native_event_t *ntv_evt;
    pe_context_t *pe_ctx = ( pe_context_t *) ctx;
    pe_control_t *pe_ctl = ( pe_control_t *) ctl;
 
@@ -1202,49 +1227,108 @@ _pe_update_control_state( hwd_control_state_t *ctl,
    /* Calling with count==0 should be OK, it's how things are deallocated */
    /* when an eventset is destroyed.                                      */
    if ( count == 0 ) {
-      SUBDBG( "Called with count == 0\n" );
+      SUBDBG( "EXIT: Called with count == 0\n" );
       return PAPI_OK;
    }
 
    /* set up all the events */
    for( i = 0; i < count; i++ ) {
       if ( native ) {
-	 /* Have libpfm4 set the config values for the event */
-	 ret=_pe_libpfm4_setup_counters(&pe_ctl->events[i].attr,
-					native[i].ni_event,
-					pe_ctx->event_table);
-	 SUBDBG( "pe_ctl->eventss[%d].config=%"PRIx64"\n",i,
-		 pe_ctl->events[i].attr.config);
-	 if (ret!=PAPI_OK) return ret;
+			// get the native event pointer used for this papi event
+			int ntv_idx = _papi_hwi_get_ntv_idx((unsigned)(native[i].ni_papi_code));
+			if (ntv_idx < -1) {
+				SUBDBG("papi_event_code: %#x known by papi but not by the component\n", native[i].ni_papi_code);
+				continue;
+			}
+			// if native index is -1, then we have an event without a mask and need to find the right native index to use
+			if (ntv_idx == -1) {
+				// find the native event index we want by matching for the right papi event code
+				for (j=0 ; j<pe_ctx->event_table->num_native_events ; j++) {
+					if (pe_ctx->event_table->native_events[j].papi_event_code == native[i].ni_papi_code) {
+						ntv_idx = j;
+					}
+				}
+			}
 
+			// if native index is still negative, we did not find event we wanted so just return error
+			if (ntv_idx < 0) {
+				SUBDBG("papi_event_code: %#x not found in native event tables\n", native[i].ni_papi_code);
+				continue;
+			}
+
+			// this native index is positive so there was a mask with the event, the ntv_idx identifies which native event to use
+			ntv_evt = (struct native_event_t *)(&(pe_ctx->event_table->native_events[ntv_idx]));
+			SUBDBG("ntv_evt: %p\n", ntv_evt);
+
+			SUBDBG("i: %d, pe_ctx->event_table->num_native_events: %d\n", i, pe_ctx->event_table->num_native_events);
+
+	    	// Move this events hardware config values and other attributes to the perf_events attribute structure
+			memcpy (&pe_ctl->events[i].attr, &ntv_evt->attr, sizeof(perf_event_attr_t));
+
+			// may need to update the attribute structure with information from event set level domain settings (values set by PAPI_set_domain)
+			// only done if the event mask which controls each counting domain was not provided
+
+			// get pointer to allocated name, will be NULL when adding preset events to event set
+			char *aName = ntv_evt->allocated_name;
+			if ((aName == NULL)  ||  (strstr(aName, ":u=") == NULL)) {
+				SUBDBG("set exclude_user attribute from eventset level domain flags, encode: %d, eventset: %d\n", pe_ctl->events[i].attr.exclude_user, !(pe_ctl->domain & PAPI_DOM_USER));
+				pe_ctl->events[i].attr.exclude_user = !(pe_ctl->domain & PAPI_DOM_USER);
+			}
+			if ((aName == NULL)  ||  (strstr(aName, ":k=") == NULL)) {
+				SUBDBG("set exclude_kernel attribute from eventset level domain flags, encode: %d, eventset: %d\n", pe_ctl->events[i].attr.exclude_kernel, !(pe_ctl->domain & PAPI_DOM_KERNEL));
+				pe_ctl->events[i].attr.exclude_kernel = !(pe_ctl->domain & PAPI_DOM_KERNEL);
+			}
+			pe_ctl->events[i].attr.exclude_guest = 1;
+
+			// libpfm4 supports mh (monitor host) and mg (monitor guest) event masks
+			// perf_events supports exclude_hv and exclude_idle attributes
+			// PAPI_set_domain supports PAPI_DOM_SUPERVISOR and PAPI_DOM_OTHER domain attributes
+			// not sure how these libpfm4 masks, perf_event attributes, and PAPI domain attributes relate to each other, the code sample below is one possibility
+//			if (strstr(ntv_evt->allocated_name, ":mg=") == NULL) {
+//				SUBDBG("set exclude_hv attribute from eventset level domain flags, encode: %d, eventset: %d\n", pe_ctl->events[i].attr.exclude_hv, !(pe_ctl->domain & PAPI_DOM_SUPERVISOR));
+//				pe_ctl->events[i].attr.exclude_hv = !(pe_ctl->domain & PAPI_DOM_SUPERVISOR);
+//			}
+
+			// set the cpu number provided with an event mask if there was one (will be -1 if mask not provided)
+			pe_ctl->events[i].cpu = ntv_evt->cpu;
+			// if cpu event mask not provided, then set the cpu to use to what may have been set on call to PAPI_set_opt (will still be -1 if not called)
+			if (pe_ctl->events[i].cpu == -1) {
+				pe_ctl->events[i].cpu = pe_ctl->cpu;
+			}
       } else {
-	  /* I'm not sure how we'd end up in this case */
-          /* should it be an error?                    */
+    	  // This case happens when called from _pe_set_overflow and _pe_ctl
+          // Those callers put things directly into the pe_ctl structure so it is already set for the open call
       }
 
-      /* Copy the inherit flag into the attribute block that will be   */
-      /* passed to the kernel */
+      // Copy the inherit flag into the attribute block that will be passed to the kernel
       pe_ctl->events[i].attr.inherit = pe_ctl->inherit;
 
       /* Set the position in the native structure */
       /* We just set up events linearly           */
       if ( native ) {
-	 native[i].ni_position = i;
+    	  native[i].ni_position = i;
+    	  SUBDBG( "&native[%d]: %p, ni_papi_code: %#x, ni_event: %#x, ni_position: %d, ni_owners: %d\n",
+			i, &(native[i]), native[i].ni_papi_code, native[i].ni_event, native[i].ni_position, native[i].ni_owners);
       }
    }
 
-   pe_ctl->num_events = count;
-   _pe_set_domain( ctl, pe_ctl->domain );
+	if (count <= skipped_events) {
+		SUBDBG("EXIT: No events to count, they all contained invalid umasks\n");
+		return PAPI_ENOEVNT;
+	}
 
-   /* actuall open the events */
+   pe_ctl->num_events = count - skipped_events;
+
+   /* actually open the events */
    /* (why is this a separate function?) */
    ret = open_pe_events( pe_ctx, pe_ctl );
    if ( ret != PAPI_OK ) {
-      SUBDBG("open_pe_events failed\n");
+      SUBDBG("EXIT: open_pe_events returned: %d\n", ret);
       /* Restore values ? */
       return ret;
    }
 
+   SUBDBG( "EXIT: PAPI_OK\n" );
    return PAPI_OK;
 }
 
@@ -1330,8 +1414,9 @@ _pe_ctl( hwd_context_t *ctx, int code, _papi_int_option_t *option )
            if (ret != PAPI_OK) {
 	      return ret;
 	   }
-	   /* looks like we are allowed, so set counting domain */
-	   return _pe_set_domain( pe_ctl, option->domain.domain );
+	   /* looks like we are allowed, so set event set level counting domains */
+       pe_ctl->domain = option->domain.domain;
+	   return PAPI_OK;
 
       case PAPI_GRANUL:
 	   pe_ctl = (pe_control_t *) ( option->granularity.ESI->ctl_state );
@@ -1625,7 +1710,7 @@ int
 _pe_shutdown_component( void ) {
 
   /* deallocate our event table */
-  _pe_libpfm4_shutdown(&perf_native_event_table);
+  _pe_libpfm4_shutdown(&_perf_event_vector, &perf_native_event_table);
 
   /* Shutdown libpfm4 */
   _papi_libpfm4_shutdown();
@@ -1914,7 +1999,7 @@ _pe_dispatch_timer( int n, hwd_siginfo_t *info, void *uc)
   }
         
   if (ioctl( fd, PERF_EVENT_IOC_DISABLE, NULL ) == -1 ) {
-      PAPIERROR("ioctl(PERF_EVENT_IOC_DISABLE) failed.\n");
+      PAPIERROR("ioctl(PERF_EVENT_IOC_DISABLE) failed");
   }
 
   if ( ( thread->running_eventset[cidx]->state & PAPI_PROFILING ) && 
@@ -2024,6 +2109,7 @@ _pe_stop_profiling( ThreadInfo_t *thread, EventSetInfo_t *ESI )
 int
 _pe_set_overflow( EventSetInfo_t *ESI, int EventIndex, int threshold )
 {
+	SUBDBG("ENTER: ESI: %p, EventIndex: %d, threshold: %d\n", ESI, EventIndex, threshold);
 
   pe_context_t *ctx;
   pe_control_t *ctl = (pe_control_t *) ( ESI->ctl_state );
@@ -2039,12 +2125,16 @@ _pe_set_overflow( EventSetInfo_t *ESI, int EventIndex, int threshold )
 	 evt_idx,EventIndex,ESI->EventSetIndex);
 
   if (evt_idx<0) {
+	SUBDBG("EXIT: evt_idx: %d\n", evt_idx);
     return PAPI_EINVAL;
   }
 
   if ( threshold == 0 ) {
     /* If this counter isn't set to overflow, it's an error */
-    if ( ctl->events[evt_idx].attr.sample_period == 0 ) return PAPI_EINVAL;
+    if ( ctl->events[evt_idx].attr.sample_period == 0 ) {
+    	SUBDBG("EXIT: PAPI_EINVAL, Tried to clear sample threshold when it was not set\n");
+    	return PAPI_EINVAL;
+    }
   }
 
   ctl->events[evt_idx].attr.sample_period = threshold;
@@ -2077,6 +2167,7 @@ _pe_set_overflow( EventSetInfo_t *ESI, int EventIndex, int threshold )
   default:
     PAPIERROR( "ctl->wakeup_mode[%d] set to an unknown value - %u",
 	       evt_idx, ctl->events[evt_idx].wakeup_mode);
+	SUBDBG("EXIT: PAPI_EBUG\n");
     return PAPI_EBUG;
   }
 
@@ -2096,6 +2187,9 @@ _pe_set_overflow( EventSetInfo_t *ESI, int EventIndex, int threshold )
     retval = _papi_hwi_start_signal( 
 				    ctl->overflow_signal, 
 				    1, ctl->cidx );
+    if (retval != PAPI_OK) {
+    	SUBDBG("Call to _papi_hwi_start_signal returned: %d\n", retval);
+    }
   } else {
     /* turn off internal overflow flag for this event set */
     ctl->overflow = 0;
@@ -2103,13 +2197,17 @@ _pe_set_overflow( EventSetInfo_t *ESI, int EventIndex, int threshold )
     /* Remove the signal handler, if there are no remaining non-zero */
     /* sample_periods set                                            */
     retval = _papi_hwi_stop_signal(ctl->overflow_signal);
-    if ( retval != PAPI_OK ) return retval;
+    if ( retval != PAPI_OK ) {
+    	SUBDBG("Call to _papi_hwi_stop_signal returned: %d\n", retval);
+    	return retval;
+    }
   }
 
   retval = _pe_update_control_state( ctl, NULL,
 				     ( (pe_control_t *) (ESI->ctl_state) )->num_events,
 				     ctx );
 
+  SUBDBG("EXIT: return: %d\n", retval);
   return retval;
 }
 
