@@ -9,8 +9,10 @@
  *  This component enables RAPL (Running Average Power Level)
  *  energy measurements on Intel SandyBridge/IvyBridge/Haswell
  *
- *  To work, the x86 generic MSR driver must be installed
- *    (CONFIG_X86_MSR) and the /dev/cpu/?/msr files must have read permissions
+ *  To work, either msr_safe kernel module from LLNL 
+ *  (https://github.com/scalability-llnl/msr-safe), or 
+ *  the x86 generic MSR driver must be installed
+ *    (CONFIG_X86_MSR) and the /dev/cpu/?/<msr_safe | msr> files must have read permissions
  */
 
 #include <stdio.h>
@@ -170,12 +172,16 @@ static int open_fd(int offset) {
   char filename[BUFSIZ];
 
   if (fd_array[offset].open==0) {
-     sprintf(filename,"/dev/cpu/%d/msr",offset);
-     fd = open(filename, O_RDONLY);
-     if (fd>=0) {
-        fd_array[offset].fd=fd;
-	fd_array[offset].open=1;
-     }
+	  sprintf(filename,"/dev/cpu/%d/msr_safe",offset);
+      fd = open(filename, O_RDONLY);
+	  if (fd<0) {
+		  sprintf(filename,"/dev/cpu/%d/msr",offset);
+          fd = open(filename, O_RDONLY);
+	  }
+	  if (fd>=0) {
+		  fd_array[offset].fd=fd;
+	      fd_array[offset].open=1;
+      } 
   }
   else {
     fd=fd_array[offset].fd;
@@ -395,11 +401,14 @@ _rapl_init_component( int cidx )
        num_read=fscanf(fff,"%d",&package);
        fclose(fff);
        if (num_read!=1) {
-	  fprintf(stderr,"error reading %s\n",filename);
+    		 strcpy(_rapl_vector.cmp_info.disabled_reason, "Error reading file: ");
+    		 strncat(_rapl_vector.cmp_info.disabled_reason, filename, PAPI_MAX_STR_LEN - strlen(_rapl_vector.cmp_info.disabled_reason) - 1);
+    		 _rapl_vector.cmp_info.disabled_reason[PAPI_MAX_STR_LEN-1] = '\0';
+    		 return PAPI_ESYS;
        }
 
        /* Check if a new package */
-       if (package < nr_cpus) {
+       if ((package >= 0) && (package < nr_cpus)) {
          if (packages[package] == -1) {
            SUBDBG("Found package %d out of total %d\n",package,num_packages);
 	   packages[package]=package;
@@ -418,9 +427,9 @@ _rapl_init_component( int cidx )
      num_cpus=j;
 
      if (num_packages==0) {
-        SUBDBG("Can't access /dev/cpu/*/msr\n");
+        SUBDBG("Can't access /dev/cpu/*/<msr_safe | msr>\n");
 	strncpy(_rapl_vector.cmp_info.disabled_reason,
-		"Can't access /dev/cpu/*/msr",PAPI_MAX_STR_LEN);
+		"Can't access /dev/cpu/*/<msr_safe | msr>",PAPI_MAX_STR_LEN);
 	return PAPI_ESYS;
      }
 
@@ -976,14 +985,14 @@ _rapl_ntv_code_to_info(unsigned int EventCode, PAPI_event_info_t *info)
 
   if ( ( index < 0) || (index >= num_events )) return PAPI_ENOEVNT;
 
-  strncpy( info->symbol, rapl_native_events[index].name, 
-	   sizeof(info->symbol));
+  strncpy( info->symbol, rapl_native_events[index].name, sizeof(info->symbol)-1);
+  info->symbol[sizeof(info->symbol)-1] = '\0';
 
-  strncpy( info->long_descr, rapl_native_events[index].description, 
-	   sizeof(info->symbol));
+  strncpy( info->long_descr, rapl_native_events[index].description, sizeof(info->long_descr)-1);
+  info->long_descr[sizeof(info->long_descr)-1] = '\0';
 
-  strncpy( info->units, rapl_native_events[index].units, 
-	   sizeof(info->units));
+  strncpy( info->units, rapl_native_events[index].units, sizeof(info->units)-1);
+  info->units[sizeof(info->units)-1] = '\0';
 
   info->data_type = rapl_native_events[index].return_type;
 
