@@ -322,6 +322,18 @@ static const intel_x86_umask_t hsw_cycle_activity[]={
      .ucntmsk= 0xf,
      .uflags = INTEL_X86_NCOMBO,
    },
+   { .uname  = "STALLS_LDM_PENDING",
+     .udesc  = "Execution stalls due to memory subsystem",
+     .ucode  = 0x0600 | (0x6 << INTEL_X86_CMASK_BIT),
+     .ucntmsk= 0xf,
+     .uflags = INTEL_X86_NCOMBO,
+   },
+   { .uname  = "CYCLES_NO_EXECUTE",
+     .udesc  = "Cycles during which no instructions were executed in the execution stage of the pipeline",
+     .ucode  = 0x0400 | (0x4 << INTEL_X86_CMASK_BIT),
+     .ucntmsk= 0xf,
+     .uflags = INTEL_X86_NCOMBO,
+   },
 };
 
 static const intel_x86_umask_t hsw_dtlb_load_misses[]={
@@ -904,6 +916,11 @@ static const intel_x86_umask_t hsw_longest_lat_cache[]={
 };
 
 static const intel_x86_umask_t hsw_machine_clears[]={
+  { .uname = "CYCLES",
+    .udesc  = "Cycles there was a Nuke. Account for both thread-specific and All Thread Nukes",
+    .ucode  = 0x100,
+    .uflags = INTEL_X86_NCOMBO,
+  },
   { .uname = "MEMORY_ORDERING",
     .udesc  = "Number of Memory Ordering Machine Clears detected",
     .ucode  = 0x200,
@@ -950,6 +967,24 @@ static const intel_x86_umask_t hsw_mem_load_uops_l3_miss_retired[]={
     .ucode  = 0x100,
     .uflags = INTEL_X86_NCOMBO | INTEL_X86_PEBS | INTEL_X86_DFL,
   },
+   { .uname  = "REMOTE_DRAM",
+     .udesc  = "Number of retired load uops that missed L3 but were service by remote RAM, snoop not needed, snoop miss, snoop hit data not forwarded (Precise Event)",
+     .ucode = 0x400,
+     .umodel = PFM_PMU_INTEL_HSW_EP,
+     .uflags= INTEL_X86_NCOMBO | INTEL_X86_PEBS,
+   },
+   { .uname  = "REMOTE_HITM",
+     .udesc  = "Number of retired load uops whose data sources was remote HITM (Precise Event)",
+     .ucode = 0x1000,
+     .umodel = PFM_PMU_INTEL_HSW_EP,
+     .uflags= INTEL_X86_NCOMBO | INTEL_X86_PEBS,
+   },
+   { .uname  = "REMOTE_FWD",
+     .udesc  = "Load uops that miss in the L3 whose data source was forwarded from a remote cache (Precise Event)",
+     .ucode = 0x2000,
+     .umodel = PFM_PMU_INTEL_HSW_EP,
+     .uflags= INTEL_X86_NCOMBO | INTEL_X86_PEBS,
+   },
 };
 
 static const intel_x86_umask_t hsw_mem_load_uops_retired[]={
@@ -1020,11 +1055,6 @@ static const intel_x86_umask_t hsw_mem_uops_retired[]={
     .ucode  = 0x2100,
     .uflags = INTEL_X86_NCOMBO | INTEL_X86_PEBS,
   },
-  { .uname = "LOCK_STORES",
-    .udesc  = "Store uops with locked access retired",
-    .ucode  = 0x2200,
-    .uflags = INTEL_X86_NCOMBO | INTEL_X86_PEBS,
-  },
   { .uname = "SPLIT_LOADS",
     .udesc  = "Line-splitted load uops retired",
     .ucode  = 0x4100,
@@ -1494,34 +1524,158 @@ static const intel_x86_umask_t hsw_offcore_response[]={
    { .uname  = "LLC_HITM",
      .udesc  = "Supplier: counts L3 hits in M-state (initial lookup)",
      .ucode = 1ULL << (18+8),
+     .umodel = PFM_PMU_INTEL_HSW,
      .grpid = 1,
    },
    { .uname  = "LLC_HITE",
      .udesc  = "Supplier: counts L3 hits in E-state",
      .ucode = 1ULL << (19+8),
+     .umodel = PFM_PMU_INTEL_HSW,
      .grpid = 1,
    },
    { .uname  = "LLC_HITS",
      .udesc  = "Supplier: counts L3 hits in S-state",
      .ucode = 1ULL << (20+8),
+     .umodel = PFM_PMU_INTEL_HSW,
      .grpid = 1,
    },
    { .uname  = "LLC_HITF",
      .udesc  = "Supplier: counts L3 hits in F-state",
      .ucode = 1ULL << (21+8),
-     .grpid = 1,
-   },
-   { .uname  = "LLC_MISS_LOCAL",
-     .udesc  = "Supplier: counts L3 misses to local DRAM",
-     .ucode = 1ULL << (22+8),
+     .umodel = PFM_PMU_INTEL_HSW,
      .grpid = 1,
    },
    { .uname  = "LLC_HITMESF",
      .udesc  = "Supplier: counts L3 hits in any state (M, E, S, F)",
      .ucode  = 0xfULL << (18+8),
      .uequiv = "LLC_HITM:LLC_HITE:LLC_HITS:LLC_HITF",
+     .umodel = PFM_PMU_INTEL_HSW,
      .grpid  = 1,
    },
+   { .uname  = "LLC_HIT",
+     .udesc  = "Alias for LLC_HITMESF",
+     .ucode  = 0xfULL << (18+8),
+     .uequiv = "LLC_HITM:LLC_HITE:LLC_HITS:LLC_HITF",
+     .umodel = PFM_PMU_INTEL_HSW,
+     .grpid  = 1,
+   },
+   { .uname  = "L3_HITM",
+     .udesc  = "Supplier: counts L3 hits in M-state (initial lookup)",
+     .ucode = 1ULL << (18+8),
+     .umodel = PFM_PMU_INTEL_HSW_EP,
+     .grpid = 1,
+   },
+   { .uname  = "L3_HITE",
+     .udesc  = "Supplier: counts L3 hits in E-state",
+     .ucode = 1ULL << (19+8),
+     .umodel = PFM_PMU_INTEL_HSW_EP,
+     .grpid = 1,
+   },
+   { .uname  = "L3_HITS",
+     .udesc  = "Supplier: counts L3 hits in S-state",
+     .ucode = 1ULL << (20+8),
+     .umodel = PFM_PMU_INTEL_HSW_EP,
+     .grpid = 1,
+   },
+   { .uname  = "L3_HITF",
+     .udesc  = "Supplier: counts L3 hits in F-state",
+     .ucode = 1ULL << (21+8),
+     .umodel = PFM_PMU_INTEL_HSW_EP,
+     .grpid = 1,
+   },
+   { .uname  = "L3_HIT",
+     .udesc  = "Supplier: counts L3 hits in M, E, S, F state",
+     .ucode = 0xfULL << (18+8),
+     .uequiv = "L3_HITM:L3_HITE:L3_HITS:L3_HITF",
+     .umodel = PFM_PMU_INTEL_HSW_EP,
+     .grpid = 1,
+   },
+   { .uname  = "L4_HIT_LOCAL_L4",
+     .udesc  = "Supplier: counts L4 hits to local L4 cache",
+     .ucode = 0x1ULL << (22+8),
+     .umodel = PFM_PMU_INTEL_HSW_EP,
+     .grpid = 1,
+   },
+   { .uname  = "L4_HIT_REMOTE_HOP0_L4",
+     .udesc  = "Supplier: counts L4 hits to remote L4 cache with 0 hop",
+     .ucode = 0x1ULL << (23+8),
+     .umodel = PFM_PMU_INTEL_HSW_EP,
+     .grpid = 1,
+   },
+   { .uname  = "L4_HIT_REMOTE_HOP1_L4",
+     .udesc  = "Supplier: counts L4 hits to remote L4 cache with 1 hop",
+     .ucode = 0x1ULL << (24+8),
+     .umodel = PFM_PMU_INTEL_HSW_EP,
+     .grpid = 1,
+   },
+   { .uname  = "L4_HIT_REMOTE_HOP2P_L4",
+     .udesc  = "Supplier: counts L4 hits to remote L4 cache with 2P hops",
+     .ucode = 0x1ULL << (25+8),
+     .umodel = PFM_PMU_INTEL_HSW_EP,
+     .grpid = 1,
+   },
+   { .uname  = "L4_HIT",
+     .udesc  = "Supplier: counts L4 hits",
+     .ucode = 0xfULL << (22+8),
+     .uequiv = "L4_HIT_LOCAL_L4:L4_HIT_REMOTE_HOP0_L4:L4_HIT_REMOTE_HOP1_L4:L4_HIT_REMOTE_HOP2P_L4",
+     .umodel = PFM_PMU_INTEL_HSW_EP,
+     .grpid = 1,
+   },
+   { .uname  = "LLC_MISS_LOCAL",
+     .udesc  = "Supplier: counts L3 misses to local DRAM",
+     .ucode = 1ULL << (22+8),
+     .umodel = PFM_PMU_INTEL_HSW,
+     .grpid = 1,
+   },
+   { .uname  = "L3_MISS_LOCAL_DRAM",
+     .udesc  = "Supplier: counts L3 misses to local DRAM",
+     .ucode = 1ULL << (26+8),
+     .umodel = PFM_PMU_INTEL_HSW_EP,
+     .grpid = 1,
+   },
+   { .uname  = "L3_MISS_LOCAL",
+     .udesc  = "Supplier: Alias to L3_MISS_LOCAL_DRAM",
+     .ucode = 1ULL << (26+8),
+     .umodel = PFM_PMU_INTEL_HSW_EP,
+     .uequiv = "L3_MISS_LOCAL_DRAM",
+     .grpid = 1,
+   },
+   { .uname  = "L3_MISS_REMOTE_HOP0_DRAM",
+     .udesc  = "Supplier: counts L3 misses to remote DRAM with 0 hop",
+     .ucode = 0x1ULL << (27+8),
+     .umodel = PFM_PMU_INTEL_HSW_EP,
+     .grpid = 1,
+   },
+   { .uname  = "L3_MISS_REMOTE_HOP1_DRAM",
+     .udesc  = "Supplier: counts L3 misses to remote DRAM with 1 hop",
+     .ucode = 0x1ULL << (28+8),
+     .umodel = PFM_PMU_INTEL_HSW_EP,
+     .grpid = 1,
+   },
+   { .uname  = "L3_MISS_REMOTE_HOP2P_DRAM",
+     .udesc  = "Supplier: counts L3 misses to remote DRAM with 2P hops",
+     .ucode = 0x1ULL << (29+8),
+     .umodel = PFM_PMU_INTEL_HSW_EP,
+     .grpid = 1,
+   },
+   { .uname  = "L3_MISS",
+     .udesc  = "Supplier: counts L3 misses to local or remote DRAM",
+     .ucode = 0xfULL << (26+8),
+     .umodel = PFM_PMU_INTEL_HSW_EP,
+     .grpid = 1,
+   },
+   { .uname  = "L3_MISS_REMOTE_DRAM",
+     .udesc  = "Supplier: counts L3 misses to remote DRAM",
+     .ucode = 0x3ULL << (27+8),
+     .umodel = PFM_PMU_INTEL_HSW_EP,
+     .grpid = 1,
+   },
+   { .uname  = "SPL_HIT",
+     .udesc  = "Supplier: counts L3 misses to remote DRAM",
+     .ucode = 0x1ULL << (30+8),
+     .umodel = PFM_PMU_INTEL_HSW_EP,
+     .grpid = 1,
+   },
    { .uname  = "SNP_NONE",
      .udesc  = "Snoop: counts number of times no snoop-related information is available",
      .ucode = 1ULL << (31+8),
@@ -1804,6 +1958,36 @@ static const intel_x86_umask_t hsw_page_walker_loads[]={
      .ucode = 0x2400,
      .uflags= INTEL_X86_NCOMBO,
    },
+   { .uname  = "EPT_DTLB_L1",
+     .udesc  = "Number of extended page table walks from the DTLB that hit in the L1D and line fill buffer",
+     .ucode = 0x4100,
+     .uflags= INTEL_X86_NCOMBO,
+   },
+   { .uname  = "EPT_ITLB_L1",
+     .udesc  = "Number of extended page table walks from the ITLB that hit in the L1D and line fill buffer",
+     .ucode = 0x8100,
+     .uflags= INTEL_X86_NCOMBO,
+   },
+   { .uname  = "EPT_DTLB_L2",
+     .udesc  = "Number of extended page table walks from the DTLB that hit in the L2",
+     .ucode = 0x4200,
+     .uflags= INTEL_X86_NCOMBO,
+   },
+   { .uname  = "EPT_ITLB_L2",
+     .udesc  = "Number of extended page table walks from the ITLB that hit in the L2",
+     .ucode = 0x8200,
+     .uflags= INTEL_X86_NCOMBO,
+   },
+   { .uname  = "EPT_DTLB_L3",
+     .udesc  = "Number of extended page table walks from the DTLB that hit in the L3",
+     .ucode = 0x4400,
+     .uflags= INTEL_X86_NCOMBO,
+   },
+   { .uname  = "EPT_ITLB_L3",
+     .udesc  = "Number of extended page table walks from the ITLB that hit in the L3",
+     .ucode = 0x8400,
+     .uflags= INTEL_X86_NCOMBO,
+   },
    { .uname  = "DTLB_MEMORY",
      .udesc  = "Number of DTLB page walker loads that hit memory",
      .ucode = 0x1800,
@@ -1814,6 +1998,16 @@ static const intel_x86_umask_t hsw_page_walker_loads[]={
      .ucode = 0x2800,
      .uflags= INTEL_X86_NCOMBO,
    },
+   { .uname  = "EPT_DTLB_MEMORY",
+     .udesc  = "Number of extended page table walks from the DTLB that hit memory",
+     .ucode = 0x4800,
+     .uflags= INTEL_X86_NCOMBO,
+   },
+   { .uname  = "EPT_ITLB_MEMORY",
+     .udesc  = "Number of extended page table walks from the ITLB that hit memory",
+     .ucode = 0x8800,
+     .uflags= INTEL_X86_NCOMBO,
+   },
 };
 
 static const intel_x86_umask_t hsw_lsd[]={
@@ -1832,6 +2026,14 @@ static const intel_x86_umask_t hsw_dsb2mite_switches[]={
    },
 };
 
+static const intel_x86_umask_t hsw_ept[]={
+   { .uname  = "CYCLES",
+     .udesc  = "Cycles for an extended page table walk",
+     .ucode = 0x0200,
+     .uflags= INTEL_X86_NCOMBO | INTEL_X86_DFL,
+   },
+};
+
 static const intel_x86_entry_t intel_hsw_pe[]={
   { .name   = "UNHALTED_CORE_CYCLES",
     .desc   = "Count core clock cycles whenever the clock signal on the specific core is running (not halted)",
@@ -2412,21 +2614,30 @@ static const intel_x86_entry_t intel_hsw_pe[]={
   { .name   = "PAGE_WALKER_LOADS",
     .desc   = "Page walker loads",
     .modmsk = INTEL_V4_ATTRS,
-    .cntmsk = 0xff,
+    .cntmsk = 0xf,
     .code = 0xbc,
     .numasks = LIBPFM_ARRAY_SIZE(hsw_page_walker_loads),
     .ngrp = 1,
     .umasks = hsw_page_walker_loads,
   },
-{ .name   = "DSB2MITE_SWITCHES",
-  .desc   = "Number of DSB to MITE switches",
-  .modmsk = INTEL_V4_ATTRS,
-  .cntmsk = 0xff,
-  .code = 0xab,
-  .numasks = LIBPFM_ARRAY_SIZE(hsw_dsb2mite_switches),
-  .ngrp = 1,
-  .umasks = hsw_dsb2mite_switches,
-},
+  { .name   = "DSB2MITE_SWITCHES",
+    .desc   = "Number of DSB to MITE switches",
+    .modmsk = INTEL_V4_ATTRS,
+    .cntmsk = 0xff,
+    .code = 0xab,
+    .numasks = LIBPFM_ARRAY_SIZE(hsw_dsb2mite_switches),
+    .ngrp = 1,
+    .umasks = hsw_dsb2mite_switches,
+  },
+  { .name   = "EPT",
+    .desc   = "Extended page table",
+    .modmsk = INTEL_V4_ATTRS,
+    .cntmsk = 0xff,
+    .code = 0x4f,
+    .numasks = LIBPFM_ARRAY_SIZE(hsw_ept),
+    .ngrp = 1,
+    .umasks = hsw_ept,
+  },
   { .name   = "OFFCORE_RESPONSE_0",
     .desc   = "Offcore response event (must provide at least one request type and either any_response or any combination of supplier + snoop)",
     .modmsk = INTEL_V4_ATTRS,
