@@ -219,7 +219,7 @@
 
 /* This is the official PAPI version */
 /* The final digit represents the patch count */
-#define PAPI_VERSION  			PAPI_VERSION_NUMBER(5,3,2,0)
+#define PAPI_VERSION  			PAPI_VERSION_NUMBER(5,4,0,0)
 #define PAPI_VER_CURRENT 		(PAPI_VERSION & 0xffff0000)
 
   /* Tests for checking event code type */
@@ -462,6 +462,7 @@ All of the functions in the PerfAPI should use the following set of constants.
 #define PAPI_2MAX_STR_LEN      256      /* For somewhat longer run-of-the-mill strings */
 #define PAPI_HUGE_STR_LEN     1024      /* This should be defined in terms of a system parameter */
 
+#define PAPI_PMU_MAX           40      /* maximum number of pmu's supported by one component */
 #define PAPI_DERIVED           0x1      /* Flag to indicate that the event is derived */
 /** @} */
 
@@ -641,6 +642,7 @@ read the documentation carefully.  */
      int hardware_intr_sig;       /**< Signal used by hardware to deliver PMC events */
 //   int opcode_match_width;      /**< Width of opcode matcher if exists, 0 if not */
      int component_type;          /**< Type of component */
+     char *pmu_names[PAPI_PMU_MAX];         /**< list of pmu names supported by this component */
      int reserved[8];             /* */
      unsigned int hardware_intr:1;         /**< hw overflow intr, does not need to be emulated in software*/
      unsigned int precise_intr:1;          /**< Performance interrupts happen precisely */
