@@ -67,9 +67,12 @@ int _papi_hwi_num_errors = 0;
 struct native_event_info {
   int cidx;
   int component_event;
+  int ntv_idx;
+  char *evt_name;
 };
 
 
+// The following array is indexed by the papi event code (after the native bit has been removed)
 static struct native_event_info *_papi_native_events=NULL;
 static int num_native_events=0;
 static int num_native_chunks=0;
@@ -78,6 +81,151 @@ char **_papi_errlist= NULL;
 static int num_error_chunks = 0;
 
 
+// pointer to event:mask string associated with last enum call to a components
+// will be NULL for non libpfm4 components
+// this is needed because libpfm4 event codes and papi event codes do not contain mask information
+char *papi_event_string = NULL;
+void
+_papi_hwi_set_papi_event_string (const char *event_string) {
+	INTDBG("event_string: %s\n", event_string);
+	if (papi_event_string != NULL) {
+		free (papi_event_string);
+		papi_event_string = NULL;
+	}
+	if (event_string != NULL) {
+		papi_event_string = strdup(event_string);
+	}
+	return;
+}
+char *
+_papi_hwi_get_papi_event_string () {
+	INTDBG("papi_event_string: %s\n", papi_event_string);
+	return papi_event_string;
+}
+void
+_papi_hwi_free_papi_event_string() {
+	if (papi_event_string != NULL) {
+		free(papi_event_string);
+		papi_event_string = NULL;
+	}
+	return;
+}
+// A place to keep the current papi event code so some component functions can fetch its value
+// The current event code can be stored here prior to component calls and cleared after the component returns
+static unsigned int papi_event_code = -1;
+static int papi_event_code_changed = -1;
+void
+_papi_hwi_set_papi_event_code (unsigned int event_code, int update_flag) {
+	INTDBG("new event_code: %#x, update_flag: %d, previous event_code: %#x\n", event_code, update_flag, papi_event_code);
+
+	// if call is just to reset and start over, set both flags to show nothing saved yet
+	if (update_flag < 0) {
+		papi_event_code_changed = -1;
+		papi_event_code = -1;
+		return;
+	}
+
+	// if 0, it is being set prior to calling a component, if >0 it is being changed by the component
+	papi_event_code_changed = update_flag;
+	// save the event code passed in
+	papi_event_code = event_code;
+	return;
+}
+unsigned int
+_papi_hwi_get_papi_event_code () {
+	INTDBG("papi_event_code: %#x\n", papi_event_code);
+	return papi_event_code;
+}
+/* Get the index into the ESI->NativeInfoArray for the current PAPI event code */
+int
+_papi_hwi_get_ntv_idx (unsigned int papi_evt_code) {
+	INTDBG("ENTER: papi_evt_code: %#x\n", papi_evt_code);
+
+	int result;
+	int event_index;
+
+	if (papi_evt_code == 0) {
+		INTDBG("EXIT: PAPI_ENOEVNT, invalid papi event code\n");
+		return PAPI_ENOEVNT;
+	}
+
+	event_index=papi_evt_code&PAPI_NATIVE_AND_MASK;
+	if ((event_index<0) || (event_index>=num_native_events)) {
+		INTDBG("EXIT: PAPI_ENOEVNT, invalid index into native event array\n");
+		return PAPI_ENOEVNT;
+	}
+
+	result=_papi_native_events[event_index].ntv_idx;
+
+	INTDBG("EXIT: result: %d\n", result);
+	return result;
+}
+
+//
+// Check for the presence of a component name or pmu name in the event string.
+// If found check if it matches this component or one of the pmu's supported by this component.
+//
+// returns true if the event could be for this component and false if it is not for this component.
+//    if there is no component or pmu name then it could be for this component and returns true.
+//
+static int
+is_supported_by_component(int cidx, char *event_name) {
+	INTDBG("ENTER: cidx: %d, event_name: %s\n", cidx, event_name);
+	int i;
+	int component_name = 0;
+	int pmu_name = 0;
+	char *wptr = NULL;
+
+	// if event does not have a component name or pmu name, return to show it could be supported by this component
+	// when component and pmu names are not provided, we just have to call the components to see if they recognize the event
+	//
+
+	// look for component names first
+	if ((wptr = strstr(event_name, ":::")) != NULL) {
+		component_name = 1;
+	} else if ((wptr = strstr(event_name, "::")) != NULL) {
+		pmu_name = 1;
+	} else {
+		INTDBG("EXIT: No Component or PMU name in event string, try this component\n");
+		// need to force all components to be called to find owner of this event
+		// ????  can we assume the default pmu when no component or pmu name is provided ????
+		return 1;
+	}
+
+	// get a temporary copy of the component or pmu name
+	int name_len = wptr - event_name;
+	wptr = strdup(event_name);
+	wptr[name_len] = '\0';
+
+	// if a component name was found, compare it to the component name in the component info structure
+	if (component_name) {
+//		INTDBG("component_name: %s\n", _papi_hwd[cidx]->cmp_info.name);
+		if (strcmp (wptr, _papi_hwd[cidx]->cmp_info.name) == 0) {
+			free (wptr);
+			INTDBG("EXIT: Component %s supports this event\n", _papi_hwd[cidx]->cmp_info.name);
+			return 1;
+		}
+	}
+
+	// if a pmu name was found, compare it to the pmu name list if the component info structure (if there is one)
+	if (pmu_name) {
+		for ( i=0 ; i<PAPI_PMU_MAX ; i++) {
+			if (_papi_hwd[cidx]->cmp_info.pmu_names[i] == NULL) {
+				continue;
+			}
+//			INTDBG("pmu_name[%d]: %p (%s)\n", i, _papi_hwd[cidx]->cmp_info.pmu_names[i], _papi_hwd[cidx]->cmp_info.pmu_names[i]);
+			if (strcmp (wptr, _papi_hwd[cidx]->cmp_info.pmu_names[i]) == 0) {
+				INTDBG("EXIT: Component %s supports PMU %s and this event\n", _papi_hwd[cidx]->cmp_info.name, wptr);
+				free (wptr);
+				return 1;
+			}
+		}
+	}
+
+	free (wptr);
+	INTDBG("EXIT: Component does not support this event\n");
+	return 0;
+}
 
 /** @internal
  * @class _papi_hwi_prefix_component_name
@@ -160,29 +308,46 @@ char *_papi_hwi_strip_component_prefix(char *event_name)
 	return (start);
 }
 
+/* find the papi event code (4000xxx) associated with the specified component, native event, and event name */
 static int
-_papi_hwi_find_native_event(int cidx, int event) {
+_papi_hwi_find_native_event(int cidx, int event, const char *event_name) {
+  INTDBG("ENTER: cidx: %x, event: %#x, event_name: %s\n", cidx, event, event_name);
 
   int i;
 
+  // if no event name passed in, it can not be found
+  if (event_name == NULL) {
+		INTDBG("EXIT: PAPI_ENOEVNT\n");
+		return PAPI_ENOEVNT;
+  }
+
   for(i=0;i<num_native_events;i++) {
-    if ((_papi_native_events[i].cidx==cidx) &&
+  	// if we have have not set up this event name yet, look at next
+  	if (_papi_native_events[i].evt_name == NULL) {
+  		continue;
+  	}
+
+  	// is this entry for the correct component and event code
+  	if ((_papi_native_events[i].cidx==cidx) &&
 	(_papi_native_events[i].component_event==event)) {
-       return i|PAPI_NATIVE_MASK;
+		// if this event name matches what we want, return its papi event code
+		if (strcmp(event_name, _papi_native_events[i].evt_name) == 0) {
+			INTDBG("EXIT: event: %#x, component_event: %#x, ntv_idx: %d, event_name: %s\n",
+				i|PAPI_NATIVE_MASK, _papi_native_events[i].component_event, _papi_native_events[i].ntv_idx, _papi_native_events[i].evt_name);
+			return i|PAPI_NATIVE_MASK;
+		}
     }
   }
 
-  return PAPI_ENOEVNT;
-
+	INTDBG("EXIT: PAPI_ENOEVNT\n");
+	return PAPI_ENOEVNT;
 }
 
 static int
-_papi_hwi_add_native_event(int event, int cidx) {
+_papi_hwi_add_native_event(int cidx, int ntv_event, int ntv_idx, const char *event_name) {
+	INTDBG("ENTER: cidx: %d, ntv_event: %#x, ntv_idx: %d, event_name: %s\n", cidx, ntv_event, ntv_idx, event_name);
 
   int new_native_event;
-
-  SUBDBG("Creating Event %#x which is comp %d internal %#x\n",
-	 num_native_events|PAPI_NATIVE_MASK,cidx,event);
   
   _papi_hwi_lock( INTERNAL_LOCK );
 
@@ -198,7 +363,13 @@ _papi_hwi_add_native_event(int event, int cidx) {
   }
 
   _papi_native_events[num_native_events].cidx=cidx;
-  _papi_native_events[num_native_events].component_event=event;
+  _papi_native_events[num_native_events].component_event=ntv_event;
+  _papi_native_events[num_native_events].ntv_idx=ntv_idx;
+  if (event_name != NULL) {
+	  _papi_native_events[num_native_events].evt_name=strdup(event_name);
+  } else {
+	  _papi_native_events[num_native_events].evt_name=NULL;
+  }
   new_native_event=num_native_events|PAPI_NATIVE_MASK;
 
   num_native_events++;
@@ -207,6 +378,7 @@ native_alloc_early_out:
 
   _papi_hwi_unlock( INTERNAL_LOCK );
 
+  INTDBG("EXIT: new_native_event: %#x, num_native_events: %d\n", new_native_event, num_native_events);
   return new_native_event;
 }
 
@@ -219,7 +391,7 @@ native_alloc_early_out:
 static int
 _papi_hwi_add_error( char *error )
 {
-	SUBDBG("Adding a new Error message |%s|\n", error);
+	INTDBG("Adding a new Error message |%s|\n", error);
 	_papi_hwi_lock(INTERNAL_LOCK);
 
 	if (_papi_hwi_num_errors >= num_error_chunks*NATIVE_EVENT_CHUNKSIZE) {
@@ -340,68 +512,75 @@ _papi_hwi_invalid_cmp( int cidx )
 
 int 
 _papi_hwi_component_index( int event_code ) {
+	INTDBG("ENTER: event_code: %#x\n", event_code);
 
   int cidx;
   int event_index;
 
-  SUBDBG("Trying to find component for native_event %#x\n",event_code);
-
   /* currently assume presets are for component 0 only */
   if (event_code&PAPI_PRESET_MASK) {
-     SUBDBG("Event %#x is a PRESET, assigning component %d\n",
-	    event_code,0);
+     INTDBG("EXIT: Event %#x is a PRESET, assigning component %d\n", event_code,0);
      return 0;
   }
 
   event_index=event_code&PAPI_NATIVE_AND_MASK;
 
   if ( (event_index < 0) || (event_index>=num_native_events)) {
-     SUBDBG("Event index %#x is out of range\n",event_index);
+     INTDBG("EXIT: Event index %#x is out of range, num_native_events: %d\n", event_index, num_native_events);
      return PAPI_ENOEVNT;
   }
 
   cidx=_papi_native_events[event_index].cidx;
 
-  SUBDBG("Found event code %d from %d, %#x\n",cidx,event_index,event_code);
-
-  if ((cidx<0) || (cidx >= papi_num_components)) return PAPI_ENOCMP;
+  if ((cidx<0) || (cidx >= papi_num_components)) {
+	  INTDBG("EXIT: Component index %#x is out of range, papi_num_components: %d\n", cidx, papi_num_components);
+	  return PAPI_ENOCMP;
+  }
 
+  INTDBG("EXIT: Found cidx: %d event_index: %d, event_code: %#x\n", cidx, event_index, event_code);
   return cidx;
 }
 
-/* Convert a component and internal event to a native_event */
+/* Convert an internal component event to a papi event code */
 int 
-_papi_hwi_native_to_eventcode(int cidx, int event_code) {
+_papi_hwi_native_to_eventcode(int cidx, int event_code, int ntv_idx, const char *event_name) {
+  INTDBG("Entry: cidx: %d, event: %#x, ntv_idx: %d, event_name: %s\n", cidx, event_code, ntv_idx, event_name);
 
   int result;
 
-  SUBDBG("Looking for component %d event %#x\n",cidx,event_code);
-
-  result=_papi_hwi_find_native_event(cidx,event_code);
-  if (result==PAPI_ENOEVNT) {
-     /* Need to allocate */
-     result=_papi_hwi_add_native_event(event_code,cidx);
+  if (papi_event_code_changed > 0) {
+	  result = _papi_hwi_get_papi_event_code();
+	  INTDBG("EXIT: papi_event_code: %#x set by the component\n", result);
+	  return result;
   }
 
+  result=_papi_hwi_find_native_event(cidx, event_code, event_name);
+  if (result==PAPI_ENOEVNT) {
+     // Need to create one
+     result=_papi_hwi_add_native_event(cidx, event_code, ntv_idx, event_name);
+  }
+
+  INTDBG("EXIT: result: %#x\n", result);
   return result;
 }
 
 /* Convert a native_event code to an internal event code */
 int
 _papi_hwi_eventcode_to_native(int event_code) {
+	INTDBG("ENTER: event_code: %#x\n", event_code);
 
   int result;
   int event_index;
 
-  SUBDBG("Looking for event for native_event %#x\n",event_code);
-
   event_index=event_code&PAPI_NATIVE_AND_MASK;
-  if (event_index>=num_native_events) return PAPI_ENOEVNT;
+ if (event_index>=num_native_events) {
+    INTDBG("EXIT: PAPI_ENOEVNT\n");
+    return PAPI_ENOEVNT;
+  }
 
   result=_papi_native_events[event_index].component_event;
   
-  SUBDBG("Found result %#x\n",result);
-
+  INTDBG("EXIT: result: %#x\n", result);
   return result;
 
 }
@@ -418,7 +597,7 @@ PAPIERROR( char *format, ... )
 	if ( ( _papi_hwi_error_level != PAPI_QUIET ) ||
 		 ( getenv( "PAPI_VERBOSE" ) ) ) {
 		va_start( args, format );
-		fprintf( stderr, "PAPI Error: " );
+		fprintf( stderr, "\nPAPI Error: " );
 		vfprintf( stderr, format, args );
 		fprintf( stderr, ".\n" );
 		va_end( args );
@@ -658,6 +837,7 @@ _papi_hwi_assign_eventset( EventSetInfo_t *ESI, int cidx )
    for( i = 0; i < max_counters; i++ ) {
       ESI->NativeInfoArray[i].ni_event = -1;
       ESI->NativeInfoArray[i].ni_position = -1;
+      ESI->NativeInfoArray[i].ni_papi_code = -1;
       ESI->NativeInfoArray[i].ni_owners = 0;
       ESI->NativeInfoArray[i].ni_bits = ((unsigned char*)ESI->NativeBits) + 
                                           (i*_papi_hwd[cidx]->size.reg_value);
@@ -841,18 +1021,26 @@ _papi_hwi_remove_EventSet( EventSetInfo_t * ESI )
      Fail,    return PAPI_ENOEVNT;
 */
 static int
-event_already_in_eventset( EventSetInfo_t * ESI, int nevt )
+event_already_in_eventset( EventSetInfo_t * ESI, int papi_event )
 {
+   INTDBG( "ENTER: ESI: %p, papi_event: %#x\n", ESI, papi_event);
    int i;
 
+   int nevt = _papi_hwi_eventcode_to_native(papi_event);
+
    /* to find the native event from the native events list */
    for( i = 0; i < ESI->NativeCount; i++ ) {
-      if ( _papi_hwi_eventcode_to_native(nevt) == 
-                                        ESI->NativeInfoArray[i].ni_event ) {
-	 INTDBG( "found native event already mapped: %#x\n", nevt );
-	 return i;
+      if ( nevt == ESI->NativeInfoArray[i].ni_event ) {
+         // Also need to check papi event code if set because the same event with different masks
+         // will generate the same libpfm4 event code (what was checked above).  But there will be
+         // different papi events created for it and they need to be handled separately.
+         if (papi_event == ESI->NativeInfoArray[i].ni_papi_code) {
+            INTDBG( "EXIT: event: %#x already mapped at index: %d\n", papi_event, i);
+            return i;
+         }
       }
    }
+   INTDBG( "EXIT: PAPI_ENOEVNT\n");
    return PAPI_ENOEVNT;
 }
 
@@ -869,12 +1057,10 @@ event_already_in_eventset( EventSetInfo_t * ESI, int nevt )
 void
 _papi_hwi_map_events_to_native( EventSetInfo_t *ESI)
 {
+    INTDBG("ENTER: ESI: %p, ESI->EventInfoArray: %p, ESI->NativeInfoArray: %p, ESI->NumberOfEvents: %d, ESI->NativeCount: %d\n", ESI, ESI->EventInfoArray, ESI->NativeInfoArray, ESI->NumberOfEvents, ESI->NativeCount);
 
     int i, event, k, n, preset_index = 0, nevt;
     int total_events = ESI->NumberOfEvents;
-
-    APIDBG("Mapping %d events in EventSet %d\n",
-	   total_events,ESI->EventSetIndex);
    
     event = 0;
     for( i = 0; i < total_events; i++ ) {
@@ -896,15 +1082,13 @@ _papi_hwi_map_events_to_native( EventSetInfo_t *ESI)
 		break;
 	     }
 
-	     APIDBG("Loking for subevent %#x\n",nevt);
+	     INTDBG("Looking for subevent %#x\n",nevt);
 
 	     /* Match each sub-event to something in the Native List */
 	     for( n = 0; n < ESI->NativeCount; n++ ) {
-	        if ( _papi_hwi_eventcode_to_native(nevt) == 
-                                 ESI->NativeInfoArray[n].ni_event ) {
-		   APIDBG("Found event %#x at position %d\n",
-			  nevt,
-			  ESI->NativeInfoArray[n].ni_position);
+	        if ( nevt == ESI->NativeInfoArray[n].ni_papi_code ) {
+	           INTDBG("Found papi event: %#x, &ESI->NativeInfoArray[%d]: %p, ni_event: %#x, ni_position %d\n",
+	              nevt, n, &(ESI->NativeInfoArray[n]), ESI->NativeInfoArray[n].ni_event, ESI->NativeInfoArray[n].ni_position);
 		   ESI->EventInfoArray[event].pos[k] = ESI->NativeInfoArray[n].ni_position;
 		   break;
 		}
@@ -915,14 +1099,16 @@ _papi_hwi_map_events_to_native( EventSetInfo_t *ESI)
        else if( IS_NATIVE(ESI->EventInfoArray[event].event_code) ) {
 	  nevt = ( int ) ESI->EventInfoArray[event].event_code;
 
-	  /* Look for the event in the NativeInfoArray */
-	  for( n = 0; n < ESI->NativeCount; n++ ) {
-	     if ( _papi_hwi_eventcode_to_native(nevt) == 
-                                  ESI->NativeInfoArray[n].ni_event ) {
-		ESI->EventInfoArray[event].pos[0] = ESI->NativeInfoArray[n].ni_position;
-		break;
-	     }
-	  }
+		// get index into native info array for this event
+		int nidx = event_already_in_eventset( ESI, nevt );
+		// if not found, then we need to return an error
+		if (nidx == PAPI_ENOEVNT) {
+			INTDBG("EXIT: needed event not found\n");
+			return;
+		}
+		ESI->EventInfoArray[event].pos[0] = ESI->NativeInfoArray[nidx].ni_position;
+		INTDBG("nidx: %d, ni_position: %d\n", nidx, ESI->NativeInfoArray[nidx].ni_position);
+
        /* It's a user-defined event */
        } else if ( IS_USER_DEFINED(ESI->EventInfoArray[event].event_code) ) {
           for ( k = 0; k < PAPI_EVENTS_IN_DERIVED_EVENT; k++ ) {
@@ -945,6 +1131,8 @@ _papi_hwi_map_events_to_native( EventSetInfo_t *ESI)
 static int
 add_native_fail_clean( EventSetInfo_t *ESI, int nevt )
 {
+	INTDBG("ENTER: ESI: %p, nevt: %#x\n", ESI, nevt);
+
    int i, max_counters;
    int cidx;
 
@@ -955,19 +1143,22 @@ add_native_fail_clean( EventSetInfo_t *ESI, int nevt )
 
    /* to find the native event from the native events list */
    for( i = 0; i < max_counters; i++ ) {
-     if ( _papi_hwi_eventcode_to_native(nevt) == ESI->NativeInfoArray[i].ni_event ) {
+//	   INTDBG("ESI->NativeInfoArray[%d]: %p, ni_event: %#x, ni_papi_event_code: %#x, ni_position: %d, ni_owners: %d\n",
+//			   i, &(ESI->NativeInfoArray[i]), ESI->NativeInfoArray[i].ni_event, ESI->NativeInfoArray[i].ni_papi_code, ESI->NativeInfoArray[i].ni_position, ESI->NativeInfoArray[i].ni_owners);
+     if ( nevt == ESI->NativeInfoArray[i].ni_papi_code ) {
 	 ESI->NativeInfoArray[i].ni_owners--;
 	 /* to clean the entry in the nativeInfo array */
 	 if ( ESI->NativeInfoArray[i].ni_owners == 0 ) {
 	    ESI->NativeInfoArray[i].ni_event = -1;
 	    ESI->NativeInfoArray[i].ni_position = -1;
+	    ESI->NativeInfoArray[i].ni_papi_code = -1;
 	    ESI->NativeCount--;
 	 }
-	 INTDBG( "add_events fail, and remove added native events "
-                 "of the event: %#x\n", nevt );
+	 INTDBG( "EXIT: nevt: %#x, returned: %d\n", nevt, i);
 	 return i;
       }
    }
+	INTDBG( "EXIT: returned: -1\n");
    return -1;
 }
 
@@ -1005,6 +1196,7 @@ static int
 add_native_events( EventSetInfo_t *ESI, unsigned int *nevt, 
                    int size, EventInfo_t *out )
 {
+	INTDBG ("ENTER: ESI: %p, nevt: %p, size: %d, out: %p\n", ESI, nevt, size, out);
    int nidx, i, j, added_events = 0;
    int retval, retval2;
    int max_counters;
@@ -1022,8 +1214,8 @@ add_native_events( EventSetInfo_t *ESI, unsigned int *nevt,
 	 /* Event is already there.  Set position */
 	 out->pos[i] = ESI->NativeInfoArray[nidx].ni_position;
 	 ESI->NativeInfoArray[nidx].ni_owners++;
-
-      } else {
+	 continue;
+      }
 
 	 /* Event wasn't already there */
 
@@ -1037,25 +1229,25 @@ add_native_events( EventSetInfo_t *ESI, unsigned int *nevt,
 	       }
 	       INTDBG( "should not happen!\n" );
 	    }
-	    INTDBG( "counters are full!\n" );
+	    INTDBG( "EXIT: counters are full!\n" );
 	    return PAPI_ECOUNT;
 	 }
-	 else {
 			
 	    /* there is an empty slot for the native event; */
 	    /* initialize the native index for the new added event */
-	    INTDBG( "Adding %#x to ESI %p Component %d\n", 
-		    nevt[i], ESI, ESI->CmpIdx );
+	    INTDBG( "Adding nevt[%d]: %#x, ESI->NativeInfoArray[%d]: %p, Component: %d\n",
+		    i, nevt[i], ESI->NativeCount, &ESI->NativeInfoArray[ESI->NativeCount], ESI->CmpIdx );
 	    ESI->NativeInfoArray[ESI->NativeCount].ni_event = 
 			  _papi_hwi_eventcode_to_native(nevt[i]);
+	    ESI->NativeInfoArray[ESI->NativeCount].ni_papi_code = nevt[i];
 
 	    ESI->NativeInfoArray[ESI->NativeCount].ni_owners = 1;
 	    ESI->NativeCount++;
 	    added_events++;
-	 }
-      }
    }
 
+   INTDBG("added_events: %d\n", added_events);
+
    /* if we added events we need to tell the component so it */
    /* can add them too.                                      */
    if ( added_events ) {
@@ -1084,18 +1276,21 @@ clean:
 		       ESI->NativeCount,
 		       context);
 	    if ( retval2 != PAPI_OK ) {
-	       PAPIERROR("update_control_state failed to re-establish "
-			 "working events!" );
+	       PAPIERROR("update_control_state failed to re-establish working events!" );
+	       INTDBG( "EXIT: update_control_state returned: %d\n", retval2);
 	       return retval2;
 	    }
+	    INTDBG( "EXIT: update_control_state returned: %d\n", retval);
 	    return retval;
 	 }
+     INTDBG( "EXIT: update_control_state returned: %d, we return: 1 (need remap)\n", retval);
 	 return 1; /* need remap */
       } else {
 	 retval = PAPI_EMISC;
 	 goto clean;
       }
    }
+   INTDBG( "EXIT: PAPI_OK\n");
    return PAPI_OK;
 }
 
@@ -1128,7 +1323,7 @@ _papi_hwi_add_event( EventSetInfo_t * ESI, int EventCode )
        return thisindex;
     }
 
-    APIDBG("Adding event to slot %d of EventSet %d\n",thisindex,ESI->EventSetIndex);
+    INTDBG("Adding event to slot %d of EventSet %d\n",thisindex,ESI->EventSetIndex);
 
     /* If it is a software MPX EventSet, add it to the multiplex data structure */
     /* and this thread's multiplex list                                         */
@@ -1295,6 +1490,7 @@ _papi_hwi_add_event( EventSetInfo_t * ESI, int EventCode )
 static int
 remove_native_events( EventSetInfo_t *ESI, int *nevt, int size )
 {
+	INTDBG( "Entry: ESI: %p, nevt: %p, size: %d\n", ESI, nevt, size);
    NativeInfo_t *native = ESI->NativeInfoArray;
    hwd_context_t *context;
    int i, j, zero = 0, retval;
@@ -1304,8 +1500,12 @@ remove_native_events( EventSetInfo_t *ESI, int *nevt, int size )
       compare to each native event in this event set,
       and decrement owners if they match  */
    for( i = 0; i < size; i++ ) {
+	int cevt = _papi_hwi_eventcode_to_native(nevt[i]);
+//	INTDBG( "nevt[%d]: %#x, cevt: %#x\n", i, nevt[i], cevt);
       for( j = 0; j < ESI->NativeCount; j++ ) {
-	 if ( native[j].ni_event ==  _papi_hwi_eventcode_to_native(nevt[i]) ) {
+	 if ((native[j].ni_event == cevt)  &&  (native[j].ni_papi_code == nevt[i]) ) {
+//		INTDBG( "native[%d]: %p, ni_papi_code: %#x, ni_event: %#x, ni_position: %d, ni_owners: %d\n", 
+//			j, &(native[j]), native[j].ni_papi_code, native[j].ni_event, native[j].ni_position, native[j].ni_owners);
 	    native[j].ni_owners--;
 	    if ( native[j].ni_owners == 0 ) {
 	       zero++;
@@ -1352,6 +1552,8 @@ remove_native_events( EventSetInfo_t *ESI, int *nevt, int size )
       }
    }
 
+	INTDBG( "ESI->NativeCount: %d, zero: %d\n", ESI->NativeCount, zero);
+
    /* to reset hwd_control_state values */
    ESI->NativeCount -= zero;
 
@@ -1484,7 +1686,7 @@ _papi_hwi_read( hwd_context_t * context, EventSetInfo_t * ESI,
 		if ( index == -1 )
 			continue;
 
-		INTDBG( "Event index %d, position is %#x\n", i, index );
+		INTDBG( "ESI->EventInfoArray: %p, pos[%d]: %d, dp[%d]: %lld\n", ESI->EventInfoArray, i, index, index, dp[index] );
 
 		/* If this is not a derived event */
 
@@ -2020,24 +2222,28 @@ _papi_hwi_derived_string( int type, char *derived, int len )
 int
 _papi_hwi_get_preset_event_info( int EventCode, PAPI_event_info_t * info )
 {
+	INTDBG("ENTER: EventCode: %#x, info: %p\n", EventCode, info);
+
 	int i = EventCode & PAPI_PRESET_AND_MASK;
 	unsigned int j;
 
 	if ( _papi_hwi_presets[i].symbol ) {	/* if the event is in the preset table */
-	   /* set whole structure to 0 */
+      // since we are setting the whole structure to zero the strncpy calls below will 
+      // be leaving NULL terminates strings as long as they copy 1 less byte than the 
+      // buffer size of the field.
 	   memset( info, 0, sizeof ( PAPI_event_info_t ) );
 
 	   info->event_code = ( unsigned int ) EventCode;
 	   strncpy( info->symbol, _papi_hwi_presets[i].symbol,
-		    sizeof(info->symbol));
+	    sizeof(info->symbol)-1);
 
 	   if ( _papi_hwi_presets[i].short_descr != NULL )
 	      strncpy( info->short_descr, _papi_hwi_presets[i].short_descr,
-				          sizeof ( info->short_descr ) );
+		          sizeof ( info->short_descr )-1 );
 
 	   if ( _papi_hwi_presets[i].long_descr != NULL )
 	      strncpy( info->long_descr,  _papi_hwi_presets[i].long_descr,
-				          sizeof ( info->long_descr ) );
+		          sizeof ( info->long_descr )-1 );
 
 	   info->event_type = _papi_hwi_presets[i].event_type;
 	   info->count = _papi_hwi_presets[i].count;
@@ -2047,17 +2253,17 @@ _papi_hwi_get_preset_event_info( int EventCode, PAPI_event_info_t * info )
 
 	   if ( _papi_hwi_presets[i].postfix != NULL )
 	      strncpy( info->postfix, _papi_hwi_presets[i].postfix,
-				          sizeof ( info->postfix ) );
+		          sizeof ( info->postfix )-1 );
 
 	   for(j=0;j < info->count; j++) {
 	      info->code[j]=_papi_hwi_presets[i].code[j];
 	      strncpy(info->name[j], _papi_hwi_presets[i].name[j],
-		      sizeof(info->name[j]));
+	      sizeof(info->name[j])-1);
 	   }
 
 	   if ( _papi_hwi_presets[i].note != NULL ) {
 	      strncpy( info->note, _papi_hwi_presets[i].note,
-				          sizeof ( info->note ) );
+		          sizeof ( info->note )-1 );
 	   }
 
 	   return PAPI_OK;
@@ -2072,16 +2278,29 @@ _papi_hwi_get_preset_event_info( int EventCode, PAPI_event_info_t * info )
 int
 _papi_hwi_query_native_event( unsigned int EventCode )
 {
+	INTDBG("ENTER: EventCode: %#x\n", EventCode);
    char name[PAPI_HUGE_STR_LEN];      /* probably overkill, */
                                       /* but should always be big enough */
    int cidx;
+   int nevt_code;
 
    cidx = _papi_hwi_component_index( EventCode );
-   if (cidx<0) return PAPI_ENOCMP;
+   if (cidx<0) {
+	   INTDBG("EXIT: PAPI_ENOCMP\n");
+	   return PAPI_ENOCMP;
+   }
 
-   return ( _papi_hwd[cidx]->ntv_code_to_name( 
-				    _papi_hwi_eventcode_to_native(EventCode), 
-				    name, sizeof(name)));
+   // save event code so components can get it with call to: _papi_hwi_get_papi_event_code()
+   _papi_hwi_set_papi_event_code(EventCode, 0);
+
+	if ((nevt_code = _papi_hwi_eventcode_to_native(EventCode)) < 0) {
+		INTDBG("EXIT: nevt_code: %d\n", nevt_code);
+		return nevt_code;
+	}
+   int ret = _papi_hwd[cidx]->ntv_code_to_name( (unsigned int)nevt_code, name, sizeof(name));
+
+   INTDBG("EXIT: ret: %d\n", ret);
+   return (ret);
 }
 
 /* Converts an ASCII name into a native event code usable by other routines
@@ -2090,12 +2309,14 @@ _papi_hwi_query_native_event( unsigned int EventCode )
 int
 _papi_hwi_native_name_to_code( char *in, int *out )
 {
+    INTDBG("ENTER: in: %s, out: %p\n", in, out);
+
     int retval = PAPI_ENOEVNT;
     char name[PAPI_HUGE_STR_LEN];	   /* make sure it's big enough */
     unsigned int i;
     int cidx;
+    char *full_event_name = strdup(in);
 
-    SUBDBG("checking all %d components\n",papi_num_components);
 	in = _papi_hwi_strip_component_prefix(in);
 
 	
@@ -2103,44 +2324,58 @@ _papi_hwi_native_name_to_code( char *in, int *out )
 
        if (_papi_hwd[cidx]->cmp_info.disabled) continue;
 
-       /* first check each component for name_to_code */
+       // if this component does not support the pmu which defines this event, no need to call it
+       if (is_supported_by_component(cidx, full_event_name) == 0) continue;
+
+       INTDBG("cidx: %d, name: %s, event: %s\n", cidx, _papi_hwd[cidx]->cmp_info.name, in);
+
+       // show that we do not have an event code yet (the component may create one and update this info)
+       // this also clears any values left over from a previous call
+       _papi_hwi_set_papi_event_code(-1, -1);
+
+       /* first check the component for name_to_code */
        retval = _papi_hwd[cidx]->ntv_name_to_code( in, ( unsigned * ) out );
-       *out = _papi_hwi_native_to_eventcode(cidx,*out);
+       if (retval==PAPI_OK) {
+	      *out = _papi_hwi_native_to_eventcode(cidx, *out, -1, in);
+	      free (full_event_name);
+	      INTDBG("EXIT: PAPI_OK  event: %s code: %#x\n", in, *out);
+	      return PAPI_OK;
+	   }
 
-       /* If not implemented, work around */
-       if ( retval==PAPI_ECMP) {
-          i = 0;
-	  _papi_hwd[cidx]->ntv_enum_events( &i, PAPI_ENUM_FIRST );
-	  
-	  //	  _papi_hwi_lock( INTERNAL_LOCK );
+		/* If not implemented, work around */
+		if ( retval==PAPI_ECMP) {
+			i = 0;
+			_papi_hwd[cidx]->ntv_enum_events( &i, PAPI_ENUM_FIRST );
 
-	  do {
-	     retval = _papi_hwd[cidx]->ntv_code_to_name(
-					  i, 
-					  name, sizeof(name));
-             /* printf("%#x\nname =|%s|\ninput=|%s|\n", i, name, in); */
-	     if ( retval == PAPI_OK && in != NULL) {
-		if ( strcasecmp( name, in ) == 0 ) {
-		   *out = _papi_hwi_native_to_eventcode(cidx,i);
-		   break;
-		} else {
-		   retval = PAPI_ENOEVNT;
+//			_papi_hwi_lock( INTERNAL_LOCK );
+
+			do {
+				// save event code so components can get it with call to: _papi_hwi_get_papi_event_code()
+				_papi_hwi_set_papi_event_code(i, 0);
+				retval = _papi_hwd[cidx]->ntv_code_to_name(
+						i,
+						name, sizeof(name));
+				/* printf("%#x\nname =|%s|\ninput=|%s|\n", i, name, in); */
+				if ( retval == PAPI_OK && in != NULL) {
+					if ( strcasecmp( name, in ) == 0 ) {
+						*out = _papi_hwi_native_to_eventcode(cidx, i, -1, name);
+			INTDBG("EXIT: PAPI_OK, event: %s, code: %#x\n", in, *out);
+						return PAPI_OK;
+					}
+					retval = PAPI_ENOEVNT;
+				} else {
+					*out = 0;
+					retval = PAPI_ENOEVNT;
+					break;
+				}
+			} while ( ( _papi_hwd[cidx]->ntv_enum_events( &i, PAPI_ENUM_EVENTS ) == PAPI_OK ) );
+
+//			_papi_hwi_unlock( INTERNAL_LOCK );
 		}
-	     } else {
-		  *out = 0;
-		  retval = PAPI_ENOEVNT;
-		  break;
-	     }
-	  } while ( ( _papi_hwd[cidx]->ntv_enum_events( &i, 
-							PAPI_ENUM_EVENTS ) ==
-					  PAPI_OK ) );
-
-	  //	  _papi_hwi_unlock( INTERNAL_LOCK );
-       }
-
-       if ( retval == PAPI_OK ) return retval;
     }
 
+    free (full_event_name);
+    INTDBG("EXIT: retval: %d\n", retval);
     return retval;
 }
 
@@ -2150,23 +2385,34 @@ int
 _papi_hwi_native_code_to_name( unsigned int EventCode, 
 			       char *hwi_name, int len )
 {
+	INTDBG("ENTER: EventCode: %#x, hwi_name: %p, len: %d\n", EventCode, hwi_name, len);
   int cidx;
   int retval; 
+  int nevt_code;
 
   cidx = _papi_hwi_component_index( EventCode );
   if (cidx<0) return PAPI_ENOEVNT;
 
   if ( EventCode & PAPI_NATIVE_MASK ) {
-	if ( (retval = _papi_hwd[cidx]->ntv_code_to_name( 
-						_papi_hwi_eventcode_to_native(EventCode), 
-						hwi_name, len) ) == PAPI_OK ) {
-    	return 
-			_papi_hwi_prefix_component_name( _papi_hwd[cidx]->cmp_info.short_name, 
-											 hwi_name, hwi_name, len);
-	} else {
-		return (retval);
+	  // save event code so components can get it with call to: _papi_hwi_get_papi_event_code()
+	  _papi_hwi_set_papi_event_code(EventCode, 0);
+
+	if ((nevt_code = _papi_hwi_eventcode_to_native(EventCode)) < 0) {
+		INTDBG("EXIT: nevt_code: %d\n", nevt_code);
+		return nevt_code;
 	}
+	if ( (retval = _papi_hwd[cidx]->ntv_code_to_name( 
+						(unsigned int)nevt_code,
+						hwi_name, len) ) == PAPI_OK ) {
+			retval = _papi_hwi_prefix_component_name( _papi_hwd[cidx]->cmp_info.short_name, 
+											 hwi_name, hwi_name, len);
+			INTDBG("EXIT: retval: %d\n", retval);
+			return retval;
+	}
+	INTDBG("EXIT: retval: %d\n", retval);
+	return (retval);
   }
+  INTDBG("EXIT: PAPI_ENOEVNT\n");
   return PAPI_ENOEVNT;
 }
 
@@ -2177,8 +2423,10 @@ int
 _papi_hwi_get_native_event_info( unsigned int EventCode,
 				 PAPI_event_info_t *info )
 {
+	  INTDBG("ENTER: EventCode: %#x, info: %p\n", EventCode, info);
     int retval;
     int cidx;
+    int nevt_code;
 
     cidx = _papi_hwi_component_index( EventCode );
     if (cidx<0) return PAPI_ENOCMP;
@@ -2186,6 +2434,8 @@ _papi_hwi_get_native_event_info( unsigned int EventCode,
     if (_papi_hwd[cidx]->cmp_info.disabled) return PAPI_ENOCMP;
 
     if ( EventCode & PAPI_NATIVE_MASK ) {
+        // save event code so components can get it with call to: _papi_hwi_get_papi_event_code()
+        _papi_hwi_set_papi_event_code(EventCode, 0);
 
        /* clear the event info */
        memset( info, 0, sizeof ( PAPI_event_info_t ) );
@@ -2199,25 +2449,33 @@ _papi_hwi_get_native_event_info( unsigned int EventCode,
        if ( retval == PAPI_ECMP ) {
 
 
-	  SUBDBG("missing NTV_CODE_TO_INFO, faking\n");
+	  INTDBG("missing NTV_CODE_TO_INFO, faking\n");
 	  /* Fill in the info structure */
 
+		if ((nevt_code = _papi_hwi_eventcode_to_native(EventCode)) < 0) {
+			INTDBG("EXIT: nevt_code: %d\n", nevt_code);
+			return nevt_code;
+		}
 	  if ( (retval = _papi_hwd[cidx]->ntv_code_to_name( 
-				    _papi_hwi_eventcode_to_native(EventCode), 
+				    (unsigned int)nevt_code,
 				    info->symbol,
 				    sizeof(info->symbol)) ) == PAPI_OK ) {
 
 	  } else {
-	     SUBDBG("failed ntv_code_to_name\n");
+	     INTDBG("EXIT: retval: %d\n", retval);
 	     return retval;
 	  }
 
+		if ((nevt_code = _papi_hwi_eventcode_to_native(EventCode)) <0) {
+			INTDBG("EXIT: nevt_code: %d\n", nevt_code);
+			return nevt_code;
+		}
 	  retval = _papi_hwd[cidx]->ntv_code_to_descr( 
-				     _papi_hwi_eventcode_to_native(EventCode), 
-                                     info->long_descr,
+				     (unsigned int)nevt_code,
+				     info->long_descr,
 				     sizeof ( info->long_descr));
 	  if (retval!=PAPI_OK) {
-	     SUBDBG("Failed ntv_code_to_descr()\n");
+	     INTDBG("Failed ntv_code_to_descr()\n");
 	  }
 
        }
@@ -2227,9 +2485,11 @@ _papi_hwi_get_native_event_info( unsigned int EventCode,
 						info->symbol, 
 						sizeof(info->symbol) );
 
+       INTDBG("EXIT: retval: %d\n", retval);
        return retval;
     }
 
+	INTDBG("EXIT: PAPI_ENOEVNT\n");
     return PAPI_ENOEVNT;
 }
 
